package org.tah.areaCenter;

import java.awt.geom.Point2D;

public class Line {

	private double x1;
	private double x2;
	private double y1;
	private double y2;
	
	public Line(double x1, double y1, double x2, double y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;
	}
	
	public Line(Point2D p1, Point2D p2) {
		this(p1.getX(), p1.getY(), p2.getX(), p2.getY());
	}
	
	public double getUx() {
		return x2 - x1;
	}
	
	public double getUy() {
		return y2 - y1;
	}
	
	public double dotProduct(Line line) {
		// Perpendicular
		double pux1 = -getUy();
		double puy1 = getUx();
		
		return ((pux1 * line.getUx()) + (puy1 * line.getUy()));
	}
	
	public double getSlope() {
		if (Utils.equals(x1, x2)) {
			return Double.MAX_VALUE;
		} else {
			return (y1 - y2) / (x1 - x2); 
		}
	}
		
	public boolean containsPoint(Point2D point) {
		return (point.getX() >= Math.min(x1, x2)
				&& point.getX() <= Math.max(x1, x2)
				&& point.getY() >= Math.min(y1, y2)
				&& point.getY() <= Math.max(y1, y2));
	}

	
	public Point2D getLineIntersection(double a, double b) {
		
		// Convert to y = ax + b
		double a1 = getSlope();
		double b1 = y1 - a1 * x1;
		
		// Ignore parallel lines
		if (Utils.equals(a1, a)) {
			return null;
		}
		
		double x;
		double y;
		
	
		// Get intersection point
		if (a1 == Double.MAX_VALUE) {
			x = x1;
			y = a * x + b;
		} else if (a == Double.MAX_VALUE) {
			x = b;
			y = a1 * x + b1;
		} else {
			x = - (b1 - b) / (a1 - a);
			y = a1 * x + b1;
		}
		
		Point2D point = new Point2D.Double(x, y);
		return ( containsPoint(point) ) ? point : null;
	}
	
	public double getLength() {
		return Math.sqrt(getUx() * getUx() + getUy() * getUy());
	}
}
