package org.tah.areaCenter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.bretth.osmosis.core.container.v0_5.BoundContainer;
import com.bretth.osmosis.core.container.v0_5.EntityContainer;
import com.bretth.osmosis.core.container.v0_5.EntityProcessor;
import com.bretth.osmosis.core.container.v0_5.NodeContainer;
import com.bretth.osmosis.core.container.v0_5.RelationContainer;
import com.bretth.osmosis.core.container.v0_5.WayContainer;
import com.bretth.osmosis.core.domain.v0_5.RelationMember;
import com.bretth.osmosis.core.domain.v0_5.Tag;
import com.bretth.osmosis.core.domain.v0_5.WayNode;
import com.bretth.osmosis.core.task.v0_5.Sink;
import com.bretth.osmosis.core.task.v0_5.SinkSource;

public abstract class PolygonFilter implements EntityProcessor, SinkSource {

	private Sink sink;
	private ArrayList<RelationContainer> relations = new ArrayList<RelationContainer>();
	private HashMap<Long, WayContainer> ways = new HashMap<Long, WayContainer>();
	private NodesCoordinates nodesCoordinates = new NodesCoordinates();
	
	protected abstract void processPolygon(MultiPolygon polygon, WayContainer way);
		
	public void process(BoundContainer bounds) {
		sink.process(bounds);
	}

	public void process(NodeContainer node) {
		nodesCoordinates.add(node.getEntity());
		sink.process(node);
	}

	public void process(WayContainer wayContainer) {
		// Save areas with > 2 nodes
		
		List<WayNode> list = wayContainer.getEntity().getWayNodeList();
		
		if (list.size() <= 2) {
			sink.process(wayContainer);
		} else { 
			ways.put(wayContainer.getEntity().getId(), wayContainer);
		}
	}

	public void process(RelationContainer relation) {
		// Save multipolygon relations
		
		for (Tag tag:relation.getEntity().getTagList()) {
			if ("type".equals(tag.getKey()) && "multipolygon".equals(tag.getValue())) {
				relations.add(relation);
				break;
			}
		}
		sink.process(relation);
	}							
				
	public void complete() {
		
		// First process all multipolygons
		for (RelationContainer relation:relations) {
			MultiPolygon polygon = new MultiPolygon(null);
			WayContainer outerWay = null;
			
			for (RelationMember member:relation.getEntity().getMemberList()) {
				if ("outer".equals(member.getMemberRole()) && polygon.getOuter() == null) {
					outerWay = ways.remove(member.getMemberId());
					if (outerWay != null) {
						Polygon p = new Polygon(outerWay.getEntity(), nodesCoordinates);
						if (p.size() > 2) {
							polygon.setOuter(p);
						}
					}
				} else if ("inner".equals(member.getMemberRole())) {
					WayContainer way = ways.get(member.getMemberId());
					if (way != null) {
						Polygon p = new Polygon(way.getEntity(), nodesCoordinates);
						if (p.size() > 2) {
							polygon.getInner().add(p);
						}
					}
				}
			}
			if (polygon.getOuter() != null) {
				processPolygon(polygon, outerWay);
				sink.process(outerWay);
			}
			
		}
		
		// Then remaining ways 
		for (WayContainer way:ways.values()) {
			Polygon outer = new Polygon(way.getEntity(), nodesCoordinates);
			if (outer.size() > 2) {
				MultiPolygon polygon = new MultiPolygon(outer);
				processPolygon(polygon, way);
				sink.process(way);
			}
		}
		
		sink.complete();
	}
	

	public void process(EntityContainer entityContainer) {
		entityContainer.process(this);
	}

	public void release() {
		sink.release();
	}

	public void setSink(Sink sink) {
		this.sink = sink;
	}

}
