package org.tah.batik;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.Socket;

public class ConnectionHandler implements Runnable {
	
	public ConnectionHandler(Socket connection) throws IOException {
		this.connection = connection;
		inputStream = new BufferedReader(new InputStreamReader(connection.getInputStream()));
		outputStream = new OutputStreamWriter(connection.getOutputStream());
		
		request = new Request();
		request.command = inputStream.readLine();
		
		String line;
		while ((line = inputStream.readLine()).trim().length() > 0) {
			String name = line.substring(0, line.indexOf('='));
			String value = line.substring(line.indexOf('=') + 1);
			request.arguments.put(name, value);
		}
	}
	
	
	private Socket connection;
	private OutputStreamWriter outputStream;
	private BufferedReader inputStream;
	private Request request;
	
	public boolean isStopCommand() {
		return request.command.equals("stop");
	}
	
	public void run() {

		StringWriter errorStringWriter = new StringWriter();
		PrintWriter errorWriter = new PrintWriter(errorStringWriter);
		try {
			if (request.command.equals("svg2png")) {
				Svg2Png svg2Png = new Svg2Png(request, outputStream, errorWriter);
				svg2Png.run();
			} else if (request.command.equals("status")) {
				// OK is enough...
			} else if (request.command.equals("stop")) {
				// OK is enough...
			} else {
				throw new Exception("Unknown command");
			}
			
			outputStream.write("OK\n");
		} catch (Throwable e) {
			try {
				e.printStackTrace(errorWriter);
				outputStream.write("ERROR\n");
				outputStream.write(errorStringWriter.toString());
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		
		try {
			if (errorStringWriter.getBuffer().length() > 0) {
				if (request.arguments.containsKey("log")) {
					PrintStream stream = new PrintStream(new File(request.arguments.get("log")));
					stream.print(errorStringWriter.toString());
					stream.close();
				}
			}
			outputStream.flush();
			outputStream.close();
			inputStream.close();
			connection.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}
}
