package org.tah.batik;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.LinkedList;
import java.util.Queue;

public class ServerMain {
	
	private static void printHelp() {
		System.out.println("Command line arguments:");
		System.out.println("  -p <number> Port on which will server listen for connections");
		System.out.println("  -b <address> The address server will bind to, default localhost");
	}
		
	public static void main(String args[]) {
		
		int port = 18123;
		String bindAddr = "localhost";
		
		Queue<String> arguments = new LinkedList<String>();
		for (String s:args) {
			arguments.add(s);
		}
		
		while (!arguments.isEmpty()) {
			String arg = arguments.poll();
			if (arg.equals("-p")) {
				try {
					port = Integer.parseInt(arguments.poll());
				} catch (NumberFormatException e) {
					System.err.println("Incorrect port number");
				}
			} else if (arg.equals("-b")) {
				bindAddr = arguments.poll();
			} else if (arg.equals("-h") || args.equals("--help")) {
				printHelp();
			} else if (arg.trim().length() == 0) {
				// 
			} else {
				System.err.println("Incorrect command line argument");
				printHelp();
				return;
			}
		}
		
        ServerSocket serverSocket = null;
        try {
            serverSocket = new ServerSocket(port, 0, InetAddress.getByName(bindAddr));
            System.out.printf("BatikAgent server is now listening on port %d\n", port);
        } catch (IOException e) {
            System.err.printf("Could not listen on port: %d\n", port);
            System.exit(-1);
        }

        while (true) {
        	try {
        		Socket connection = serverSocket.accept();
        		ConnectionHandler handler = new ConnectionHandler(connection);
        		boolean stop = handler.isStopCommand();
        		new Thread(handler).start();
        		if (stop) {
        			System.out.println("Stopping server...");
        			return;
        		}
        	} catch (IOException e) {
        		e.printStackTrace();
        	}
        }
	}
}
