package org.tah.batik;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.batik.apps.rasterizer.SVGConverter;
import org.apache.batik.apps.rasterizer.SVGConverterException;
import org.apache.batik.apps.rasterizer.SVGConverterFileSource;
import org.apache.batik.apps.rasterizer.SVGConverterSource;
import org.apache.batik.transcoder.Transcoder;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.TranscodingHints;
import org.apache.batik.transcoder.image.ImageTranscoder;
import org.apache.batik.transcoder.image.PNGTranscoder;

public class Svg2Png {
	
	public static final String ARGUMENT_AREA = "area";
	public static final String ARGUMENT_HEIGHT = "height";
	public static final String ARGUMENT_WIDTH = "width";
	public static final String ARGUMENT_SOURCE = "source";
	public static final String ARGUMENT_DESTINATION = "destination";
	
	public Svg2Png(Request request, Writer outputStream, PrintWriter errorWriter) throws IOException {
		this.request = request;
		this.errorWriter = errorWriter;
	}
	
	private Request request;
	private PrintWriter errorWriter;
	
	public void run() throws Exception {
		
        // Now, get the transcoder to use for the operation
        Transcoder transcoder = new PNGTranscoder();

        // Now, compute the set of transcoding hints to use
        Map<TranscodingHints.Key, Object> map = new HashMap<TranscodingHints.Key, Object>();

        if (request.arguments.containsKey(ARGUMENT_AREA)) {
        	// Set AOI. ----------------------------------------------------------
        	StringTokenizer tokenizer = new StringTokenizer(request.arguments.get(ARGUMENT_AREA), ",");		
        	map.put(org.apache.batik.transcoder.image.ImageTranscoder.KEY_AOI, 
        			new Rectangle2D.Float(Float.parseFloat(tokenizer.nextToken()), Float.parseFloat(tokenizer.nextToken()), Float.parseFloat(tokenizer.nextToken()), Float.parseFloat(tokenizer.nextToken())));
        }


        // Set image height and width. ----------------------------------------
        if (request.arguments.containsKey(ARGUMENT_WIDTH)) {
        	map.put(ImageTranscoder.KEY_WIDTH, Float.parseFloat(request.arguments.get(ARGUMENT_WIDTH)));	
        }
        if (request.arguments.containsKey(ARGUMENT_HEIGHT)) {
        	map.put(ImageTranscoder.KEY_HEIGHT, Float.parseFloat(request.arguments.get(ARGUMENT_HEIGHT)));
        }
        
        transcoder.setTranscodingHints(map);
        transcoder.setErrorHandler(new MyErrorHandler(errorWriter));
        
        SVGConverterSource inputFile = new SVGConverterFileSource(new File(request.arguments.get(ARGUMENT_SOURCE)));
        File outputFile  = new File(request.arguments.get(ARGUMENT_DESTINATION));

                
        TranscoderInput input = null;
        TranscoderOutput output = null;
        OutputStream outputStream = null;

        input = new TranscoderInput(inputFile.getURI());

        try {
        	outputStream = new FileOutputStream(outputFile);
        } catch(FileNotFoundException fnfe) {
        	throw new SVGConverterException(SVGConverter.ERROR_CANNOT_OPEN_OUTPUT_FILE,
        			new Object[] {outputFile.getName()});
        }
        output = new TranscoderOutput(outputStream);

        try { 
        	transcoder.transcode(input, output);
        } finally {
       		outputStream.flush();
       		outputStream.close();        	
        }       	
	}
}
